@@ -1,6 +1,7 @@
+#!/usr/bin/perl -w
 use strict;
 $^W = 1;
-use Socket qw ( inet_aton );
+use Socket qw ( inet_aton inet_ntoa );
 use IO::File;
 print "Building registry... this will take a moment...\n";
 
@@ -10,7 +11,6 @@ for (my $i=0; $i<=31; $i++){
 }
 
 # this is our fast stash
-my %ip;
 my $tree = IPTree->new();
 
 # and this is our pre-generated list of ranges
@@ -24,9 +24,7 @@ while (my $line = <REG>){
     $cc = 'UK' if ($cc eq 'GB');
     my $packed_ip = inet_aton($ip);
     my $packed_range = substr(pack('N',$log2{$size}),3,1);
-    my $key = $packed_ip.$packed_range;
-    $tree->add($key,$cc);
-    $ip{$key} = $cc;
+    $tree->add($packed_ip,$packed_range,$cc);
 }
 close REG || warn("can't close $reg_file, but continuing: $!");
 
@@ -60,17 +58,18 @@ use strict;
 use Socket qw ( inet_aton inet_ntoa );
 $^W = 1;
 
-my $null;
 my @mask;
 my %ctod;
 my @dtoc;
 my $bit0;
 my $bit1;
+my $bits12;
+my $null;
 BEGIN {
-    $null = inet_aton ('0.0.0.0');
     $bit0 = substr(pack('N',2 ** 31),0,1);
-    $bit1 = substr(pack('N',2 ** 31),0,1);
-
+    $bit1 = substr(pack('N',2 ** 30),0,1);
+    $bits12 = substr(pack('N',2 ** 30 + 2 ** 29),0,1);
+    $null = substr(pack('N',0),0,1);
     for (my $i = 1; $i <= 32; $i++){
 	$mask[$i] = pack('N',2 ** (32 - $i));
     }
@@ -88,22 +87,18 @@ sub new ()
     }, 'IPTree';
 }
 
-sub add ($$$)
+sub add ($$$$)
 {
-    my ($tree,$key,$cc) = @_;
+    my ($tree,$ip,$packed_range,$cc) = @_;
     $tree->_ccPlusPlus($cc);
-    my $ip = substr($key,0,4);
-    my $netmask = 32 - $ctod{substr($key,4,1)};
-#    printf "%2s - %15s (",$cc, inet_ntoa($ip);
+    my $netmask = 32 - $ctod{$packed_range};
     for (my $i = 1; $i <= $netmask; $i++){
 	if (($ip & $mask[$i]) eq $mask[$i]){
-#	    print '1';
 	    unless (exists $tree->{1}){
 		$tree->{1} = {};
 	    }
 	    $tree = $tree->{1};
 	} else {
-#	    print '0';
 	    unless (exists $tree->{0}){
 		$tree->{0} = {};
 	    }
@@ -111,7 +106,6 @@ sub add ($$$)
 	}
     }
     $tree->{cc} = $cc;
-#    print ")\n";
 }
 
 sub get_cc_as_num ($)
@@ -124,7 +118,6 @@ sub get_cc_as_num ($)
 		       keys %{$self->{countries}})
 	{
 	    $self->{sorted_cc}->{$c} = $i;
-#	    print "$c - $i\n";
 	    $i++;
 	}
     }
@@ -156,7 +149,6 @@ sub _ccPlusPlus ($)
 sub printTree ($)
 {
     my ($self,$fh) = @_;
-    $self->_findSize($self);
     _printSize($self,$self,$fh);
 }
 
@@ -164,19 +156,15 @@ sub _printSize
 {
     my ($self,$node,$fh) = @_;
     if (exists $node->{cc}){
-	# country codes are two bytes - might shrink this to one and a bit
+	# country codes are one or two bytes - 
+	# popular codes being stored in one byte
 	my $cc = $self->get_cc_as_num($node->{cc});
-	if ($cc < 64){
-	    print $fh ($dtoc[$cc] | $bit0);
-	} else {
-	    print $fh $dtoc[255] . $dtoc[$self->get_cc_as_num($node->{cc})];
-	}
+	$cc = _encode_cc($cc);
+	print $fh $cc;
     } else {
 	# jump distances are three bytes - might also be shrunk later
-	my $jump = $node->{0}->{size};
-	my $binary_jump = substr(pack('N',$jump),1,3);
-	die("bad jump: $jump") if ($jump >= 2 ** 24);
-	die("bad jump: $jump") if (($binary_jump & $bit0) eq $binary_jump);
+	my $jump = _findSize($self,$node->{0});
+	my $binary_jump = _encode_size($jump);
 	print $fh $binary_jump;
 
 	_printSize($self,$node->{0},$fh);
@@ -184,21 +172,33 @@ sub _printSize
     }
 }
 
+sub _encode_cc
+{
+    my $cc = shift;
+    if ($cc < 64){
+	return $dtoc[$cc] | $bit0;
+    } else {
+	return $dtoc[255] . $dtoc[$cc];
+    }
+}
+
+sub _encode_size
+{
+    my $size = shift;
+    return substr(pack('N',$size),1,3);
+}
+
 sub _findSize
 {
     my ($self,$node) = @_;
     my $size = 0;
     if (exists $node->{cc}){
 	my $cc = $self->get_cc_as_num($node->{cc});
-	if ($cc < 64){
-	    $size = 1;
-	} else {
-	    $size = 2;
-	}
+	$size = length(_encode_cc($cc));
     } else {
-	$size = 3 + $self->_findSize($node->{0}) + $self->_findSize($node->{1});
+	my $child_size = $self->_findSize($node->{0}) + $self->_findSize($node->{1});
+	$size = $child_size + length(_encode_size($child_size));
     }
-    $node->{size} = $size;
     return $size;
 }
 
